// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
#include "OmImage.h"
#include <Dal.h>
#include <iostream>

/****************************************************
* LoadImage
* Loads the image from the fiven file name
****************************************************/
float** LoadImage(char *fileName, int &nx, int &ny)
{
    cout << "Loading image line 11" << fileName << endl;
    DataSet* set = dataSetServer->open(fileName, Dal::Read);
    Array* array = set->array("PRIMARY");
    longVec arraySize = array->size();
    nx = arraySize[1];
    ny = arraySize[0];
    cout << "actual image size =nx=" << nx << " ny=" << ny << endl;
    
    ArrayData *arrayData = array->data();
    real32 * data = arrayData->real32Data();

    float min = 1e6;
    float max = -min;

    float **image = NULL;
    //    allocate(&image, nx, ny); 

    for (int i = 0; i < nx; i++)
        for (int j = 0; j < ny; j++)
        {
        image[i][j] =  data[i * nx + j];
        if (data[i * nx + j] < min) 
            min = data[i * nx + j];
        else
    	    if (data[i * nx + j] > max) max = data[i * nx + j];
         
    }
    cout << "min=" << min << " max=" << max << endl;
    delete arrayData;
    dataSetServer->close(set);
    return image;
}



/****************************************************
* LoadImage
* Loads the image from the given file betweem
* xl-xu and yl - yu
*
****************************************************/
float** LoadImage(char *fileName, int &nx, 
           int &ny, int xl, int xu, int yl, int yu)
{
    cout << "Loadimage line 52  " << fileName << endl;
  
    DataSet *set = dataSetServer->open(fileName, Dal::Read);
    Array* array = set->array("PRIMARY");
    longVec arraySize = array->size();
    int nx1 = (int)arraySize[1];
    int ny1 = (int)arraySize[0];

    cout << "actual image sizes - arraysize0=" <<  arraySize[0] << endl;
    cout << "arraysize1=" <<  arraySize[1] << endl;

    xl = max(0, xl);
    xu = min(xu, nx1 - 1);
    yl = max(0, yl);
    yu = min(yu, ny1 - 1);

    cout << "xl=" << xl << " xu=" << xu << endl;
    cout << "yl=" << yl << " yu=" << yu << endl;

    nx = xu - xl + 1;
    ny = yu - yl + 1;

    cout << "output image size = nx=" << nx << " ny=" << ny << endl;

     
    ArrayData *arrayData = array->data();
    real32 * data = arrayData->real32Data();

  

    float **image = NULL;
    //    allocate(&image, nx, ny); 

    cout << "Line 77\n";


    for (int i = 0; i < nx; i++)
        for (int j = 0; j < ny; j++)
        {
	    image[i][j] =  data[i + xl + (j + yl) * nx1];
  

           
         
    }
  
    delete arrayData;
    dataSetServer->close(set);

    CreateImage("cbtempimage.fit", image, nx, ny);

    return image;
}

/****************************************************
* CreateImage
*
****************************************************/

void CreateImage(const char *fileName, float **image, int nx, int ny)
{
  
  cout << "In OmImage.cc createimage Creating file " << fileName << endl;
  cout << "nx=" << nx << " ny=" << ny << endl;  
  remove(fileName);
  cout << "Line 116\n";
 
   return;
  //  const string name = "cbimage.fits";
  DataSet * set  = dataSetServer->open(fileName, Dal::Create);
  // dataSetServer->close(set);
 
  // DataSet *set  = dataSetServer->open(name, Dal::Create);

 cout << "Line 119\n";
  
 longVec arraySize;
 cout << "Line 122\n";
 
  arraySize.push_back((long)ny);
  arraySize.push_back((long)nx);


  cout << "Line 128\n";

  Array *array  = set->addArray("PRIMARY",
				Dal::Real32,
				 arraySize);

  //    DataSet::addArray(const std::string&, Dal::DataType, const longVec&, const 
  //    std::string& = "", const std::string& = "", int = -1)


  *(set->addAttribute("PIX_ENCL","number of pixels enclosed"))
      = 100;

 



  ArrayData * arrayData = array->data();
  real32 *data = arrayData->real32Data();


 cout << "Line 149\n";

  int k = 0;
  for (int i  = 0; i < nx; i++)
    {
      for (int j = 0; j < ny; j++)
	{
	  data[k++] = image[i][j];
	}
    }
  delete data;
  

  
  dataSetServer->close(set);



}
  


