// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
#ifndef IMAGEDLG_H
#define IMAGEDLG_H
#include <qcombobox.h>
#include <qpushbutton.h>
#include <qpixmap.h>
#include <qimage.h>
#include <qlabel.h>
#include <qregexp.h>
#include <qapplication.h>
#include <qdatetime.h>
#include <qpainter.h>
#include <qwmatrix.h>
#include <qdialog.h>
#include <qscrollview.h>
#include <vector>
#include <qrect.h>
#include <qcursor.h>
#include <string>
using std::string;
class QImage;
class QPoint;
class QPen;
class QPointArray;
class QPixmap;
class QLabel;
class QTable;
class QSpinBox;
class QPainter;
class QCheckBox;
class OmSourceButton;
class OmSourceLabel;
class QRect;
class QPoint;
class QScrollView;
class ImageView; // Displays the image, etc
class QTimer;
class ImageDlg;
class ExtractionRegion;
class QTime;
class QPixmap;

class ExtractionParameters;

/*********************************************************
*
*********************************************************/
class ImageView : public QWidget
{
  Q_OBJECT

    public slots:
     void paintEvent( QPaintEvent * );
       
     void setScrollPos(int x, int y) 
     {

         _scrollPos.setX(x); 
         _scrollPos.setY(y); 
  
     }; // Sets the scroll position

    void initialiseImage();
    void resizeLineBuffer();
  
    protected:
 
    void        mousePressEvent( QMouseEvent * );
    void        mouseReleaseEvent( QMouseEvent * );
    void        mouseMoveEvent( QMouseEvent * );
    void keyPressEvent( QKeyEvent *e);
      

    QPoint mouseToLogical(QPoint);     
    float mouseToOMX(float x);
    float mouseToOMY(float y);
 
    public:
        ImageView(ImageDlg *parent); 
	~ImageView() {};
        void handleReleaseLeftButton(const QPoint &point);
        void handleReleaseRightButton(const QPoint &point);
      

        static void extractSpectrumCoordinates(float &xZero, float &yZero, 
					       float &xFirst, float &yFirst, float &width,
					  float &leftOffset, float &rightOffset,
					  float &leftWidth, float &rightWdith);




        QPoint _oldPoint;
    
    private:
        QPoint centroidPoint(const QPoint &omPoint, int &height, float &, float &);

        void viewSpectrum();
           void SetRegion(const float &x0, const float &y0,
		      const float &xFirst, const float &yFirst, ExtractionRegion &region);   
     

     
        void handleReleaseMidButton(const QPoint &point);
     
        QPoint RefinePosition(const QPoint &point);
 

        ImageDlg *_parent;    
        

        int _binY;
       
        QRect _imageRect;
        int _imageX;
        int _imageY;
        int _height;
        
        QPoint _scrollPos;     

        QPixmap _pixMap;
        QPen _cyanPen;
        QPen _magentaPen;
        QPen _redPen;  
        QPen _blackPen;
        QPen _bluePen;
       
        QPixmap _buffer;
        QPixmap _lineBuffer;
        QPixmap _graphBuffer;
        

	//        QPoint _oldPoint;
        bool _eraseLineBuffer;
        int _noSpectra;   
        int _noSpectraSelected;
        QTime _time;     
       
};

/********************************************************
*
*
*
********************************************************/
class ImageDlg : public QDialog
{
    Q_OBJECT
      signals:
  
    void repaintImage(QPaintEvent *);
    void initialiseImage();
    void resizeLineBuffer();
    void displayInformation1();
  
   
public:
    ImageDlg( QWidget *parent, vector<ExtractionRegion>* extractionRegionList, 

             int nx, int ny, int binY, string filter, 
             ExtractionParameters *extractionParameters, const char *name=0 );
    ~ImageDlg();    
    int getNoSpectra() {return _noSources;};

public slots:


 private slots:
   
    void deleteList();    
    void quit();    
    void okay();
    void displayInformation();  
    void widthHasChanged(int);    
    void heightHasChanged(int);
    void selectionHasChanged(int, int);         
    void leftBackgroundOffsetHasChanged(int);
    void leftBackgroundWidthHasChanged(int);
    void rightBackgroundOffsetHasChanged(int);
    void rightBackgroundWidthHasChanged(int);
    void extractionModeHasChanged(int);
    void showHelpInformation();    
    void changeCmap(int index);
    void changeScale(int index);
    void changeImageSize(int size);
    void changeImageView();
    void changeImage();
    void drawVerticalLine(bool );

    void drawCentralRect(bool );

    void drawLeftBackgroundRect(bool );
    void drawRightBackgroundRect(bool );
   
    void centroidyZero(bool);   
    void centroidxFirst(bool);   
    void changeExtractionParameters();

protected:
    void paintEvent( QPaintEvent * );
    void mousePressEvent( QMouseEvent * );
    void mouseReleaseEvent( QMouseEvent * );
    void mouseMoveEvent( QMouseEvent * );
    QPoint mouseToLogical(QPoint);     
    void keyPressEvent( QKeyEvent *e);
 
 private:
    void resizeImageView();
    void loadTable(); 
    void centreCursor() { QCursor::setPos((_scrollView->geometry().right() - 
                    _scrollView->geometry().left()) / 2,
		    _scrollView->geometry().bottom() - 50); }; 

     
public:
    void updateTable(ExtractionRegion *region);
    void drawGraph(QPainter* painter, QString title = QString(" "));
    void computeProfile();      // Computes the profile 
    void getMinMax();
    void loadImage(); 
  
    bool runOmgrism(int il, int iu);
    bool runOmgrism(ExtractionRegion *region);

    // The following is for conveting a pixel position
    // on the image to an om pixel position 
    QPoint convertToOmPoint(QPoint &point)
    {
        int x = int(double(point.x()) * _xScale);
        int y = int(double(_image.height() - point.y()) * _yScale);
        return QPoint(x, y);

    }

    // The following is for conveting an OM pixel
    // to a pixel position on the image 
    QPoint convertFromOmPoint(QPoint &point)
    {
        int x = int(double(point.x()) / _xScale);
        int y = _image.height() - int(double(point.y()) / _yScale);
        return QPoint(x, y);
    }

    vector<ExtractionRegion>* _extractionRegionList;

    //    vector<QImage>* _spectrumPlotList;
    vector<ExtractionRegion>  _newExtractionRegionList;
    vector<QPixmap> _graphList;
      
    int _nx, _ny;  // The actual no of pixels in the OM image
    int _binY;
    int _centroidDy;
    string _filter;
    ExtractionParameters* _extractionParameters;  
  
    QImage _image; 
    QRect _imageRect;
    QRect _imagePixelRect;
    QRect _graphRect;
    QPixmap _buffer; 
    QPoint _tempvalue; 

    int _imageX;
    int _imageY;
  
    QPixmap _pixMap;
  
    QPen _blackPen;
    QPen _bluePen;

    QRect _rect;
    QRect _leftRect;
    QRect _rightRect;
    int _width;
    int _height;
    int _omWidth;
    int _omHeight;

    int _sourceNo;
  
    QPoint _point;
   
    int _leftBackgroundOffset;
    int _leftBackgroundWidth;
    int _rightBackgroundOffset;
    int _rightBackgroundWidth;

    int _omLeftBackgroundOffset;
    int _omLeftBackgroundWidth;
    int _omRightBackgroundOffset;
    int _omRightBackgroundWidth;

    QRect _omCentralRect;
    QRect _omLeftBackgroundRect;
    QRect _omRightBackgroundRect;

 


    int _noSources;
    int _regionNo;   
    int _regionStartSize;  
    int _scaleChecked;
    int _cmapChecked;
    int _zoomFactor;        
    int _extractionMode;


    int _ninX, _ninY;
    float _minX, _maxX;
    float _minY, _maxY;
    int _size;
    float *_x, *_y;
    float _xScale, _yScale;
    float _backgroundMode;
   
    // OmSourceBrowser::ObsDetails* obsDetails,  
    QTable* _table;
    OmSourceLabel* _coordLabel;
    OmSourceLabel* _noSourcesLabel;
    //    OmSourceLabel* _regionNoLabel;
                 
  
    OmSourceButton *_deleteListBtn;
    OmSourceButton *_changeImageBtn;;



    // Widgets for the spectrum extraction region
    QLabel* _widthLabel;
    QLabel* _heightLabel;

    QSpinBox* _widthSpinBox;
    QSpinBox* _heightSpinBox;





    // Widgets for the background extraction region
    OmSourceLabel* _leftBackgroundOffsetLabel;   // Distance from left-hand edge of extraction region
    OmSourceLabel* _leftBackgroundWidthLabel;
    
    QSpinBox* _leftBackgroundOffsetSpinBox;
    QSpinBox* _leftBackgroundWidthSpinBox;
   
   
    OmSourceLabel* _rightBackgroundOffsetLabel; // Distance from right-hand edge of extraction region
    OmSourceLabel* _rightBackgroundWidthLabel;
    
    QSpinBox* _rightBackgroundOffsetSpinBox;
    QSpinBox* _rightBackgroundWidthSpinBox;
 

    // Widgets for omgrism extraction mode

    OmSourceLabel* _extractionModeLabel;
  
    QSpinBox* _extractionModeSpinBox;
 

    OmSourceLabel* _imageSizeLabel;
  

    QSpinBox* _imageSizeSpinBox;
    QCheckBox* _drawVerticalLineCheckBox;
  
    QCheckBox* _drawCentralRectCheckBox;
    QCheckBox* _drawLeftBackgroundRectCheckBox;
    QCheckBox* _drawRightBackgroundRectCheckBox;
    QCheckBox* _centroidyZeroCheckBox;
    QCheckBox* _centroidxFirstCheckBox;
  

    QScrollView* _scrollView;
    ImageView* _imageView;   

    QTimer* _timer;
    bool _drawVerticalLine;
 
    bool _drawCentralRect;
    bool _drawLeftBackgroundRect;
    bool _drawRightBackgroundRect;
    bool _drawRegions;


    bool _centroidyZero, _centroidxFirst;
    QPoint _mousePoint;
         
};
#endif





