// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
#include <qtooltip.h>
#include <qstring.h>
#include <qlabel.h>
#include <qwidget.h>
#include "OmSourceLabel.h"
#include <qfont.h>
#include <qfontinfo.h>
#include <qfontmetrics.h>


OmSourceLabel::OmSourceLabel(QWidget *parent, 
       QFont &font1, QString title, 
       int x, int y, QString help) :
       QLabel(title, parent)
{
    
    setFont(font1);
    QFont f = font();
   
   
    QFontMetrics fm( f );
 
    int pixelWidth = fm.width(title ) + 5;
    _width = pixelWidth;
       
    setGeometry(x, y, pixelWidth + 2, fm.height() + 2);


    //    setFrameStyle( QFrame::Panel | QFrame::Raised );
    setFrameStyle( QFrame::Panel | QFrame::Sunken );
   
    setAlignment( Qt::AlignHCenter | Qt::AlignVCenter );
    //    setPaletteBackgroundColor(QColor(255, 0, 0));    
    //    setPaletteBackgroundColor(QColor(0, 0, 255));    
   setPaletteBackgroundColor(QColor(156, 156, 156));    

    if (help)   
        QToolTip::add(this, help);

};




/************************************************************************
*
*
************************************************************************/


OmSourceLabel::OmSourceLabel(QWidget *parent, QString title, 
       int x, int y, int height, QString help) :
       QLabel(title, parent)
{
    
    QFont f = font();
   
   
    QFontMetrics fm( f );
 
    int pixelWidth = fm.width(title ) + 5;
    
          
    setGeometry(x, y, pixelWidth, height);


    //    setFrameStyle( QFrame::Panel | QFrame::Raised );
    setFrameStyle( QFrame::Panel | QFrame::Sunken );
   
    setAlignment( Qt::AlignHCenter | Qt::AlignVCenter );
    //    setPaletteBackgroundColor(QColor(255, 0, 0));    
    //    setPaletteBackgroundColor(QColor(0, 0, 255));    
   setPaletteBackgroundColor(QColor(156, 156, 156));    

    if (help)   
        QToolTip::add(this, help);


    _width = pixelWidth;

};

/**************************************************************
*
*
**************************************************************/
OmSourceLabel::OmSourceLabel(QWidget *parent, QString title, 
       int x, int y, int width, int height, QString help) :
       QLabel(title, parent)
{
    
   
   
    setGeometry(x, y, width, height);
    
    _width = width;
    //    setFrameStyle( QFrame::Panel | QFrame::Raised );
    setFrameStyle( QFrame::Panel | QFrame::Sunken );
   
    setAlignment( Qt::AlignHCenter | Qt::AlignVCenter );
    
    //    setPaletteBackgroundColor(QColor(0, 0, 255));    
    setPaletteBackgroundColor(QColor(156, 156, 156));    

    if (help)   
        QToolTip::add(this, help);

    setText(title, width);
    

};
/**************************************************************
*
*
**************************************************************/
void OmSourceLabel::setText(QString &title, int length)
{
    
    QFont f = font();
   
   
    QFontMetrics fm( f );
 
    int pixelWidth = 0;

    if(length < 0)
        pixelWidth = fm.width(title ) + 3;
    else
        pixelWidth = length * fm.width("a") + 3;

       
    pixelWidth += 2;
    int x = this->x();
    int y = this->y();

    int height = this->height();   
 
    setGeometry(x, y, pixelWidth, height);
    

    //    setFrameStyle( QFrame::Panel | QFrame::Raised );
    setFrameStyle( QFrame::Panel | QFrame::Sunken );
   
    setAlignment( Qt::AlignHCenter | Qt::AlignVCenter );
    //    setPaletteBackgroundColor(QColor(255, 0, 0));    
    //    setPaletteBackgroundColor(QColor(0, 0, 255));    
   setPaletteBackgroundColor(QColor(156, 156, 156));    
   QLabel::setText(title);
    

};

/***************************************************************
*
****************************************************************/
void OmSourceLabel::setText(char *text, int length)
{
    
    QString title(text);
    QFont f = font();
   
   
    QFontMetrics fm( f );
 
    int pixelWidth = 0;
    if(length < 0)
        pixelWidth = fm.width(title ) + 3;
    else
      pixelWidth = length * fm.width("a") + 3;
    

    pixelWidth += 2;
    int x = this->x();
    int y = this->y();
    int height = this->height();   
 
     
    setGeometry(x, y, pixelWidth, height);
    


    //    setFrameStyle( QFrame::Panel | QFrame::Raised );
    setFrameStyle( QFrame::Panel | QFrame::Sunken );
   
    setAlignment( Qt::AlignHCenter | Qt::AlignVCenter );
    //    setPaletteBackgroundColor(QColor(255, 0, 0));    
    //    setPaletteBackgroundColor(QColor(0, 0, 255));    
   setPaletteBackgroundColor(QColor(156, 156, 156));    
   QLabel::setText(title);
    

};









