! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
! -*-f90-*-
!
! Author:    jbakker
! Generated: Tue Sep 17 15:37:05 CEST 2002
! Usage:     Mecahnism to report progress to User via the GUI interface
! Info:      Some computations may take a while. The User may interpret this
!            as a crash of the program. In order to avoid this confusion,
!            we provide a mechanism to notify theat the process is still 
!            running.
!            
! Example:   use omprogress_mod
!            type(OmProgressT) :: p
!            integer           :: i
!            
!            call construct(p,label="What am I doing", steps=100)
!            do i=1,totalSteps(p)
!              ! do some computations here
!              ...
!            
!              if (i == 50) then
!                 call setLabel(p,"change the text halfway")
!              endif
!            
!              ! and show the progress
!              call setProgress(p,step=i)
!            end do
!            
!            call destruct(p)
module omprogress_mod
   use omprogress_t
   implicit none

   private ! all, but:
   public  :: OmProgressT
   public  :: construct
   public  :: destruct
   public  :: setProgress
   public  :: setLabel
   public  :: progress
   public  :: totalsteps


   interface construct
      module procedure construct_
   end interface

   interface destruct
      subroutine omprogress_mod_delete(t)
         use omprogress_t
         type(OmProgressT),intent(inout)  :: t
      end subroutine omprogress_mod_delete
   end interface

   interface setProgress
      subroutine omprogress_mod_setprogress(t,step)
         use omprogress_t
         type(OmProgressT),intent(inout) :: t
         integer,          intent(in)    :: step
      end subroutine omprogress_mod_setprogress
   end interface

   interface setLabel
      subroutine omprogress_mod_setlabel(t,label)
         use omprogress_t
         type(OmProgressT),intent(inout) :: t
         character(*),     intent(in)    :: label
      end subroutine omprogress_mod_setlabel
   end interface

   interface progress
      function omprogress_mod_progress(t) result(steps)
         use omprogress_t
         type(OmProgressT),intent(in)    :: t
         integer                         :: steps
      end function omprogress_mod_progress
   end interface

   interface totalSteps
      function omprogress_mod_totalsteps(t) result(total)
         use omprogress_t
         type(OmProgressT),intent(in)    :: t
         integer                         :: total
      end function omprogress_mod_totalsteps
   end interface

contains
   subroutine construct_(t,label,steps)
      type(OmProgressT),intent(inout) :: t
      character(*),     intent(in)     :: label
      integer,          intent(in)     :: steps
      interface
         function omprogress_mod_new(label,steps) result(t)
            use omprogress_t
            character(*),     intent(in)     :: label
            integer,          intent(in)     :: steps
            type(OmProgressT)                :: t
         end function omprogress_mod_new
      end interface
      t=omprogress_mod_new(label,steps)
   end subroutine construct_
end module omprogress_mod





